#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-rgma-servicetool-config v. 5.2.7
#
# Post-installation script for configuring the gLite RGMA Service Publisher Service
# Joachim Flammer <Joachim.Flammer@cern.ch>
#
# Version info: $Id: glite-rgma-servicetool-config.py,v 1.38.2.16 2006/05/02 10:20:56 flammer Exp $
#
# Usage: python glite-rgma-servicetool-config [-c|-v|-h|--help|--service]
#        -c                          print configuration
#        -v                          print version
#        -h,--help                   print usage info
#        --verbose                   turn verbosity on
#
#        --start                     start Service Publisher
#        --stop                      stop Service Publisher
#        --status                    print service status (0 = running, 1 = idle)
#        --configure                 configure Service Publisher
#        --subservice                use script as subservice script
#
#        --siteconfig=URL            specify URL of the site configuration file
#
#        --addExternalServices       configure servicetool to publish external services.
#                                    This requires a configuration file
#                                      "%s/etc/config/glite-rgma-servicetool-external.cfg.xml" %  os.environ['GLITE_LOCATION']  
#                                    with the corresponding entries (see installation guide for details).
#        --removeService=serviceName remove service publishing for service with 'serviceId' from publishing
#
# Return codes: 0 - Ok
#               1 - Failed
#
#####################################################################################

import os,string,socket,pwd
import sys
import getopt
import commands

from gLiteInstallerLib import gLib
from gLiteInstallerLib import ConfigParams
from gliteRgmaServicetool import gliteRgmaServicetool

# Set global variables here 
global params                # all config values from the XML file

class glite_rgma_servicetool:

    #################################################################
    # init
    #
    # init Service Publisher
    #
    #################################################################
    def __init__(self):
        
        self.verbose = 0
        self.version = "5.2.7"
        self.name = "glite-rgma-servicetool"
        self.friendly_name = "gLite R-GMA Service Publisher"


    #################################################################
    # banner
    #
    #################################################################
    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    

    #################################################################
    # copyright
    #
    #################################################################
    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'


    #################################################################
    # version
    #
    #################################################################
    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    

    #################################################################
    # usage
    #
    # print usage information
    #
    #################################################################
    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print "Usage: \n Edit the configuration file %s.cfg.xml in" % self.name,
        print "%s/etc.config/templates\n" % os.environ['GLITE_LOCATION'],
        print "save it as %s/etc/config/%s.cfg.xml" %(os.environ['GLITE_LOCATION'], self.name),
        print "and run the script as follows\n",
        print "python %s-config [OPTION...]""" % self.name

        print '    -c, --checkconf              print the service configuration'
        print '    -v, --version                print the version of the configuration script'
        print '    -h, --help                   print this usage information'
        print '    --verbose                    turn verbosity on'
        print ''
        print '    --start                      start %s' % self.friendly_name
        print '    --stop                       stop %s' % self.friendly_name
        print '    --status                     print service status (0 = running, 1 = idle)'
        print '    --configure                  configure %s' % self.friendly_name
        print ''
        print '    --siteconfig=URL             URL of the site configuration file'
        print ''
        print '    --configureExternalServices  configure servicetool to publish external services.'
        print '                                 This requires a configuration file'
        print '                                      %s/etc/config/glite-rgma-servicetool-external.cfg.xml' %  os.environ['GLITE_LOCATION']
        print '                                 with the corresponding entries (see installation guide for details).'
        print "    --removeService=serviceId    remove service publishing for service with 'serviceId' from publishing"
        print ''
        print '    --subservice                 use script as subservice script'
        print '\n'


    #################################################################
    # configure
    #
    # configure Service Publisher (and service configruation file (if option given)
    #
    # return 0 ok
    #        1 error
    #################################################################
    def configure(self):
                            
        # ------------------------------------------------------------
        # Check if Service Publisher should be activated
        # ------------------------------------------------------------                

        if params.has_key('rgma.servicetool.activate'):
            self.activated = params['rgma.servicetool.activate']
        else:
            glib.printErrorMessage("Parameter \"rgma.servicetool.activate\" not found. Please check the configuration file.")
            return 1
        
        if (self.activated == 'false'):
            glib.printInfoMessage("%s not activated." % service.friendly_name)
            return 0
        else:
            if (self.activated != 'true'):
                glib.printErrorMessage("Parameter \"rgma.servicetool.activate\" has an incorrect value. It must be (true|false). Please check the configuration file.")
                return 1            
        
        # ------------------------------------------------------------
        # Get and check corresponding configuration values
        # ------------------------------------------------------------        
    
        print "Checking configuration for %s  " % service.friendly_name,        
        
        # subservice (internal)               
        if params.has_key('rgma.servicetool.run_as_subservice'):
            self.subservice = params['rgma.servicetool.run_as_subservice']
        else:
            self.subservice = "false"
                
        # rgma siteId (user)
        if params.has_key('rgma.servicetool.siteId'):
            self.siteId = params['rgma.servicetool.siteId']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.servicetool.siteId\" not found. Please check configuration file.")
            return 1
                    
        # rgma server hostname (user)
        if params.has_key('rgma.server.hostname'):
            self.server_hostname = params['rgma.server.hostname']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.server.hostname\" not found. Please check configuration file.")
            return 1
                    
        # rgma schema hostname (user)
        if params.has_key('rgma.schema.hostname'):
            self.schema_hostname = params['rgma.schema.hostname']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.schema.hostname\" not found. Please check configuration file.")
            return 1
                                
        # rgma registry hostname: at least one must be given (user)
        if not len(params['rgma.registry.hostname']) > 0:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.registry.hostname\" not found. Please correct configuration file.")
            return 1
            
        # host certificate file (advanced)
        if params.has_key('host.certificate.file'):
            self.host_certificate_file = params['host.certificate.file']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"host.certificate.file\" not found.")
            return 1
            
        # host.key.file (advanced)
        if params.has_key('host.key.file'):
            self.host_key_file = params['host.key.file']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"host.key.file\" not found.")
            return 1
            
        # ca.certificates.dir (advanced)
        if params.has_key('ca.certificates.dir'):
            self.ca_certificates_dir = params['ca.certificates.dir']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"ca.certificates.dir\" not found.")
            return 1    
            
        # user certificates path (advanced)
        if params.has_key('user.certificate.path'):
            self.user_certificates_path = params['user.certificate.path']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"user.certificate.path\" not found.")
            return 1
            
        # rgma user name (advanced)
        if params.has_key('rgma.user.name'):
            self.rgma_user_name = params['rgma.user.name']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.user.name\" not found.")
            return 1
            
        # rgma user group (advanced)
        if params.has_key('rgma.user.group'):
            self.rgma_user_group = params['rgma.user.group']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.user.group\" not found.")
            return 1
            
        # rgma secure mode (advanced)
        if params.has_key('rgma.secure.mode'):
            self.rgma_secure_mode = params['rgma.secure.mode']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.secure.mode\" not found.")
            return 1    
            
        glib.printOkMessage()

        # temporary message as long as only one registry is allowed
        if len(params['rgma.registry.hostname']) > 1:
            print
            glib.printWarningMessage("You specified more than one registry.")
            glib.printWarningMessage("At the moment only one registry is allowed.")
            glib.printWarningMessage("Configuration will use first specified registry '%s'" % params['rgma.registry.hostname'][0])
            glib.printWarningMessage("and ignore the others.")
            print
    
        # ------------------------------------------------------------        
        # Check certificates (if enabled and not run as subservice)
        # ------------------------------------------------------------    

        if subservice=="false":
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if glib.check_certs(params) != 0:
                        return 1
    
        # ------------------------------------------------------------
        # Run security util script (if not run as subservice)
        # ------------------------------------------------------------
                
        if subservice=="false":
            print "Configuring security ...                                 "
            errorcode = os.system("%s/etc/config/scripts/glite-security-utils-config.py --subservice" % os.environ['GLITE_LOCATION'])

            if errorcode:
                print "Configuring security                                      ",
                glib.printFailedMessage()
                return 1
        
            print "Configuring security                                      ",
            glib.printOkMessage()
            
        #-------------------------------------------------------------------------------
        # Verify the $GLITE_LOCATION/etc directory structure
        #-------------------------------------------------------------------------------
        
        if self.verbose:
            print "Verifying the %s/etc directory structure          " % os.environ['GLITE_LOCATION'],
    
        path = '%s/etc' % os.environ['GLITE_LOCATION']
        if not os.path.exists(path):
            print "Verifying the %s/etc directory structure          " % os.environ['GLITE_LOCATION'],            
            glib.printFailedMessage()
            glib.printErrorMessage('%s/etc does not exist! Please verify your gLite installation' % os.environ['GLITE_LOCATION'])
            return 1
            
        path = '%s/etc/rgma-servicetool' % os.environ['GLITE_LOCATION']
        if not os.path.exists(path):
            print "Verifying the %s/etc directory structure          " % os.environ['GLITE_LOCATION'],            
            glib.printFailedMessage()
            glib.printErrorMessage('%s/etc/rgma-servicetool does not exist! Please verify your gLite installation' % os.environ['GLITE_LOCATION'])
            return 1                    
        
        if self.verbose:
            glib.printOkMessage()
        
        #-------------------------------------------------------------------------------
        # Verify the JAVA installation
        #-------------------------------------------------------------------------------
        
        if glib.checkJava() != 0:
            return 1        
        
        # ------------------------------------------------------------
        # configure R-GMA
        # ------------------------------------------------------------    

        if self.configureRgma():
            return 1

        # ------------------------------------------------------------
        # configure R-GMA client
        # ------------------------------------------------------------    

        if self.configureRgmaClient():
            return 1
        if self.configureRgmaClientSecurity():
            return 1
        
        # ------------------------------------------------------------
        # configure R-GMA client security
        # ------------------------------------------------------------        

        if self.configureRgmaServicetool():
            return 1
        
        # ------------------------------------------------------------
        # registering R-GMA servicetool (if not run as subservice)
        # ------------------------------------------------------------
        
        if subservice=="false":
            glib.registerService()
            
        return 0
                            

    #################################################################
    # configureRgmaServicetool
    #
    # configure RGMA servicetool specific settings
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configureRgmaServicetool(self):

        # ------------------------------------------------------------
        # set siteId for R-GMA service-tool
        # ------------------------------------------------------------
        
        path = '%s/etc/rgma-servicetool/servicetool.conf' % os.environ['GLITE_LOCATION']

        # check if we can write to file
        try:
            file = open(path, 'w')
            file.write("site=%s\n" % self.siteId)
            file.close()
        except IOError, ex:
            if ex.strerror == 'Permission denied':
                print "Configuring %s                      " % service.friendly_name,
                glib.printFailedMessage()
                glib.printErrorMessage("File '%s' is not writable" % path)
                return 1
            else:
                raise ex
        
        if self.verbose:
            print "Configuring %s                 " % service.friendly_name,
            glib.printOkMessage()                                   

        # ------------------------------------------------------------
        # make subdirectory services and
        # move init.d script to $GLITE_LOCATION/etc/ and change rights
        # ------------------------------------------------------------

        if not os.path.exists('%s/etc/rgma-servicetool/services' % os.environ['GLITE_LOCATION']):
            if os.makedirs('%s/etc/rgma-servicetool/services' % os.environ['GLITE_LOCATION']):
                print "Creating directories and setting permissions              ",
                glib.printFailedMessage()
                glib.printErrorMessage("Could not create directory '%s/etc/rgma-servicetool/services'" % os.environ['GLITE_LOCATION'])
                return 1
        
        if self.verbose:
            print "Creating directories and setting permissions              ",
            glib.printOkMessage()        
                                
        return 0

    
    #################################################################
    # configureRgma
    #
    # configure RGMA
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configureRgma(self):
            
        registries_string = ""
        
        if self.verbose:
            print "\nConfiguring R-GMA ..."
        else:
            print "Configuring R-GMA                                         ",        

# the following line should be put back as soon as multiple registries are enabled again            
####
#        for item in params['rgma.registry.hostname']:
#
#            registries_string = registries_string + " --registry %s" % item                        
#            if self.verbose:
#                print "Setting up R-GMA to use registry server '%s'" % item
####

        # the following lines should be removed as soon as multiple registries are enabled again
        # start-new-codeblock
        registries_string = "--registry %s" % params['rgma.registry.hostname'][0]
        if self.verbose:
            print "Setting up R-GMA to use registry server '%s'" % params['rgma.registry.hostname'][0]
        # end new-code-block
        
        errorcode = 0
        errorcode = glib.export('RGMA_HOME', os.environ['GLITE_LOCATION'])

        if errorcode:
            if self.verbose:
                print "Configuring R-GMA                                         ",
            glib.printFailedMessage()
            glib.printErrorMessage("Could not export RGMA_HOME environment variable.")
            return 1
        
        # decide on security setting
        if self.rgma_secure_mode=="true":
            secure_setting="--secure=yes"
        else:
            secure_setting="--secure=no"
        
        # add server, schema and registry(ies)                                
        errorcode = 0
        if self.verbose == 0:
            errorcode += os.system('python %s/share/rgma/scripts/rgma-setup.py --server %s --schema %s %s %s --silent'  \
                % (os.environ['GLITE_LOCATION'], self.server_hostname, self.schema_hostname, registries_string, secure_setting))
        else:
            errorcode += os.system('python %s/share/rgma/scripts/rgma-setup.py --server %s --schema %s %s %s'  \
                % (os.environ['GLITE_LOCATION'], self.server_hostname, self.schema_hostname, registries_string, secure_setting))
        
        if errorcode:
            if self.verbose:
                print "Configuring R-GMA                                         ",                    
            glib.printFailedMessage()
            return 1

        if self.verbose:
            print "Configuring R-GMA                                         ",        

        glib.printOkMessage()    
        return 0
    
    
    #################################################################
    # configureRgmaClient
    #
    # configure R-GMA client
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configureRgmaClient(self):
    
        # ------------------------------------------------------------
        # create rgma group and user
        # ------------------------------------------------------------

        if self.verbose == 0:
            print "Creating R-GMA user and group                             ",
        else:
            print "Creating R-GMA user and group ...                         "
        
        if glib.add_user(self.rgma_user_name,self.rgma_user_group) == -1:
            glib.printFailedMessage()
            return 1

        if self.verbose == 0:            
            glib.printOkMessage()
        else:
            print "Creating R-GMA user and group                             ",
            glib.printOkMessage()

        # ------------------------------------------------------------
        # Updating host key and certificate for R-GMA
        # ------------------------------------------------------------

        self.rgma_user_home_dir = pwd.getpwnam(self.rgma_user_name)[5]

        self.user_cert_filename = "hostcert.pem"
        self.user_key_filename  = "hostkey.pem"

        # create dir user_certificates_path if it does not exist
        if not os.path.exists("%s/%s" % (self.rgma_user_home_dir, self.user_certificates_path)):
            os.mkdir("%s/%s" % (self.rgma_user_home_dir, self.user_certificates_path),0755)

        # copy key and cert file        
        errorcode = 0
        errorcode += os.system("cp %s %s/%s/%s" % \
            (self.host_key_file, self.rgma_user_home_dir, self.user_certificates_path, self.user_key_filename))/256
        errorcode += os.system("cp %s %s/%s/%s" % \
            (self.host_certificate_file, self.rgma_user_home_dir, self.user_certificates_path, self.user_cert_filename))/256
        if errorcode:
            print "Updating hostkeys and certificates for R-GMA              ",
            glib.printFailedMessage()
            glib.printErrorMessage("Could not copy certificate and key for user '%s'" % self.rgma_user_name)
            glib.printErrorMessage("to files '%s/%s/%s' and '%s/%s/%s'" % \
                (self.rgma_user_home_dir, self.user_certificates_path, self.user_cert_filename, \
                  self.rgma_user_home_dir, self.user_certificates_path, self.user_key_filename))
            return 1
        
        # change owner of key and cert file        
        errorcode = 0
        errorcode += os.system("chown %s:%s %s/%s/%s" % \
            (self.rgma_user_name, self.rgma_user_group, \
             self.rgma_user_home_dir, self.user_certificates_path, self.user_key_filename))/256
        errorcode += os.system("chown %s:%s %s/%s/%s" % \
            (self.rgma_user_name, self.rgma_user_group, \
             self.rgma_user_home_dir, self.user_certificates_path, self.user_cert_filename))/256
        
        if errorcode:
            print "Updating hostkeys and certificates for R-GMA              ",
            glib.printFailedMessage()
            glib.printErrorMessage("Could not change access rights of certificate and key for user '%s:%s'" % \
                (self.rgma_user_name, self.rgma_user_group))
            glib.printErrorMessage("for file '%s/%s/hostkey.pem' and '%s/%s/hostcert.pem'" % \
                (self.rgma_user_home_dir, self.user_certificates_path))
            return 1

        if self.verbose:
            print "Updating hostkeys and certificates for R-GMA              ",
            glib.printOkMessage()

        return 0

    #################################################################
    # configureRgmaClientSecurity
    #
    # configure R-GMA server security
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configureRgmaClientSecurity(self):

        print "Configuring R-GMA client security                         ",
        
        path = "%s/etc/rgma/ClientAuthentication.props" % os.environ['GLITE_LOCATION']

        try:
            file = open(path, 'w')
            file.write('sslCertFile=%s/%s/%s\n' % (self.rgma_user_home_dir, self.user_certificates_path, self.user_cert_filename))
            file.write('sslKey=%s/%s/%s\n' % (self.rgma_user_home_dir, self.user_certificates_path, self.user_key_filename))
            file.write('crlFiles=%s/*.r0\n' % self.ca_certificates_dir)
            file.write('sslCAFiles=%s/*.0\n' % self.ca_certificates_dir)
            file.write('#gridProxyFile=/tmp/grid-user-proxy.pem\n')
            file.close()
        except IOError, ex:
            if ex.strerror == 'Permission denied':
                glib.printFailedMessage()
                glib.printErrorMessage("File '%s' is not writable" % path)
                return 1
            else:
                glib.printFailedMessage()
                raise ex
                return 1

        glib.printOkMessage()                                
        return 0

    
    #################################################################
    # start
    #
    # start Service Publisher
    #
    # return 0 ok
    #        1 error
    #################################################################
    def start(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Starting the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
        # ------------------------------------------------------------
        # check activation status
        # ------------------------------------------------------------
        # activation
        if params.has_key('rgma.servicetool.activate'):
            activated = params['rgma.servicetool.activate']
        else:
            glib.printErrorMessage("Parameter \"rgma.servicetool.activate\" not found. Please check configuration file.")
            return 1
        
        if (activated == 'false'):
            glib.printInfoMessage("%s not activated." % service.friendly_name)
            glib.printInfoMessage("%s will not be started." % service.friendly_name)
            return -1
        else:
            if (activated != 'true'):
                glib.printErrorMessage("Parameter \"rgma.servicetool.activate\" has incorrect value. Must be (true|false). Please check configuration file.")
                return 1

        # ------------------------------------------------------------
        # start R-GMA Service Publisher
        # ------------------------------------------------------------
        
        # check status of R-GMA Service Publisher
        status_output = commands.getstatusoutput("%s/etc/init.d/rgma-servicetool status" % os.environ['GLITE_LOCATION'])
        status = status_output[0]/256
        output = status_output[1]
            
        if status==0:
            
            # Service Publisher is running
            glib.printInfoMessage("%s is already running." % service.friendly_name)
            glib.printInfoMessage("Stopping and restarting %s ..." % service.friendly_name)
            
            # stop servicetool first
            errorcode = os.system("%s/etc/init.d/rgma-servicetool stop" % os.environ['GLITE_LOCATION'])/256
            if errorcode:
                glib.printErrorMessage("Stopping %s failed." % service.friendly_name)
                return 1

        else:
            if status !=1:       
                glib.printWarningMessage("\n%s is in unknown status. Trying to start %s ..." % (service.friendly_name, service.friendly_name))
                glib.printWarningMessage("Status script returned: %s" % output)

        servicetool_lock_file = "/var/lock/subsys/rgma-servicetool"
        if os.path.exists(servicetool_lock_file):
            glib.printWarningMessage("rgma-servicetool lock file exists at")
            glib.printWarningMessage("   %s" % servicetool_lock_file)
            glib.printWarningMessage("It looks like the application has not been stopped cleanly.")                
            glib.printWarningMessage("Please have a look at the log files.")
            glib.printWarningMessage("Lock file will be removed to start R-GMA Service Publisher.")
            os.system("rm %s" % servicetool_lock_file)

        print "Starting %s                    " % service.friendly_name,

        # start R-GMA Service Publisher            
        errorcode = os.system("%s/etc/init.d/rgma-servicetool start" % os.environ['GLITE_LOCATION'])/256
        if errorcode:
            glib.printFailedMessage()
            return 1

        glib.printOkMessage()    

        return 0        
        

    #################################################################
    # stop
    #
    # stop Service Publisher
    #
    # return 0 ok
    #        1 error
    #################################################################
    def stop(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Stopping the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
        # ------------------------------------------------------------
        # stop R-GMA Service Publisher
        # ------------------------------------------------------------
        
        # check status of R-GMA Service Publisher
        status_output = commands.getstatusoutput("%s/etc/init.d/rgma-servicetool status" % os.environ['GLITE_LOCATION'])
        status = status_output[0]/256
        output = status_output[1]
            
        if status==1:
            # Service Publisher is not running
            glib.printInfoMessage("%s is not running." % service.friendly_name)
            return -1
        
        else:
            if status !=0:       
                glib.printWarningMessage("%s is in unknown status. Trying to stop %s ..." % (service.friendly_name, service.friendly_name))
                glib.printWarningMessage("Status script returned: %s" % output)

        print "Stopping %s                    " % service.friendly_name,

        # stop Service Publisher            
        errorcode = os.system("%s/etc/init.d/rgma-servicetool stop" % os.environ['GLITE_LOCATION'])/256
        if errorcode:
            glib.printFailedMessage()
            return 1

        glib.printOkMessage()

        return 0    
                

    #################################################################
    # status
    #
    # print status of Service Publisher
    #
    # return 0         running
    #        1         stopped
    #        other     errorcode
    #################################################################
    def status(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Checking status of %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
        
        # rgma run servicetool
        if params.has_key('rgma.servicetool.activate'):
            run_servicetool = params['rgma.servicetool.activate']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.servicetool.activate\" not found.")
            return 1        
                
        if run_servicetool=="true":
            
            # ------------------------------------------------------------
            # print status of rgma Service Publisher
            # ------------------------------------------------------------

            status_output = commands.getstatusoutput("%s/etc/init.d/rgma-servicetool status" % os.environ['GLITE_LOCATION'])
            status = status_output[0]/256
            output = status_output[1]
        
            if status==1:            
                print "\nThe %s is not running." % service.friendly_name
                return 1
            else:
                if status == 0:
                    print "\nThe %s is running." % service.friendly_name
                    return 0
                else:
                    glib.printWarningMessage("%s is in unknown state." % service.friendly_name)
                    glib.printWarningMessage("Status script returned: %s" % output)
                    return 1
                    
        else:
            # servicetool is not supposed to be running
            return 0 
                        

    #################################################################
    # removeService
    #
    # remove service from Service Publisher
    #
    # params serviceId  service id to be removed
    #
    # return 0         ok 
    #        1         error
    #################################################################
    def removeService(self,serviceId):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Removing service from %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
                        
        rgmaServicetool = gliteRgmaServicetool()
        if rgmaServicetool.removeServiceFile(serviceId):
            return 1

        return 0           
 
                
    #################################################################
    # addExternalServices
    #
    # add external services to Service Publisher
    #
    # return 0         ok 
    #        1         error
    #################################################################
    def addExternalServices(self):
        
        glib.params = params
        rgmaServicetool = gliteRgmaServicetool()
        if rgmaServicetool.configureServices(glib):
            return 1
            
        return 0

    #################################################################
    # showConfiguration
    #
    # show configuration
    #
    # return 0         ok 
    #        1         error
    #################################################################
    def showConfiguration(self):
        
        self.copyright()
        self.showVersion()
        glib.print_params(params)

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "             Main settings of %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        print "  R-GMA SERVER"
        print "     --------------------------------------------------------------"
        if params.has_key('rgma.server.hostname'):
            print "     \033[34m[Location]\x1b[0m          %s " % params['rgma.server.hostname']
        else:
            glib.printErrorMessage("Parameter 'rgma.server.hostname' not found.")
        print "     --------------------------------------------------------------"
        print
        
        # print schema settings
        print "  R-GMA SCHEMA SERVER"
        print "     --------------------------------------------------------------"
        if params.has_key('rgma.schema.hostname'):
            print "     \033[34m[Location]\x1b[0m          %s " % params['rgma.schema.hostname']
        else:
            glib.printErrorMessage("Parameter 'rgma.schema.hostname' not found.")        
        print "     --------------------------------------------------------------"
        print
        
        # print registry settings
        print "  R-GMA REGISTRY SERVER"        
        print "     --------------------------------------------------------------"
# The following lines should be put in place as soon as multiple registries in enabled again
######
#        if params.has_key('rgma.registry.hostname'):
#            for item in params['rgma.registry.hostname']: 
#                print "     \033[34m[Location]\x1b[0m          %s " % item
#        else:
#            glib.printErrorMessage("Parameter 'rgma.registry.hostname' not found.")        
######

        # The following lines can be removed as soon as multiple registries are enabled again
        # start-new-block
        if params.has_key('rgma.registry.hostname'):
            print "     \033[34m[Location]\x1b[0m          %s " % params['rgma.registry.hostname'][0]
        else:
            glib.printErrorMessage("Parameter 'rgma.registry.hostname' not found.")    
        
        if params.has_key('rgma.registry.hostname'):
            if len(params['rgma.registry.hostname']) > 1:
                print
                glib.printWarningMessage("You specified more than one registry.")
                glib.printWarningMessage("At the moment only one registry is allowed.")
                glib.printWarningMessage("Configuration will use first specified registry '%s'" % params['rgma.registry.hostname'][0])
                glib.printWarningMessage("and ignore the others.")
                print        
        # end-new-block
            
        print "     --------------------------------------------------------------"
        print
        
        print "  R-GMA Servicetool"
        print "     --------------------------------------------------------------"
        if params.has_key('rgma.secure.mode'):
            print "     \033[34m[SecureMode]\x1b[0m        %s " % params['rgma.secure.mode']
        else:
            glib.printErrorMessage("Parameter 'rgma.secure.mode' not found.")   
        if params.has_key('rgma.servicetool.siteId'):
            print "     \033[34m[SiteId]\x1b[0m            %s " % params['rgma.servicetool.siteId']
        else:
            glib.printErrorMessage("Parameter 'rgma.servicetool.siteId' not found.")   

        print "     --------------------------------------------------------------"        
        print
    
        # print configured services
        rgmaServicetool = gliteRgmaServicetool()
        if rgmaServicetool.showServices():
            sys.exit(1)
        

#################################################################
# loadDefaultParameters
#
# load the default parameters
#
# return 0 ok
#        1 error
#################################################################
def loadDefaultParameters(params):
   
    # rgma default params
    params['rgma.servicetool.activate']         = 'true'
    params['rgma.servicetool.status_interval']  = '60'   # seconds
    params['rgma.servicetool.publish_interval'] = '3600' # seconds
    params['rgma.servicetool.url_wsdl']         = 'not available'
    params['rgma.servicetool.url_semantics']    = 'not available'        
    params['rgma.user.name']                    = "rgma"
    params['rgma.user.group']                   = "rgma"
    params['rgma.secure.mode']                  = "true"
    
    return 0

#################################################################
# set_env
#
# Set all environment variables
#################################################################
def set_env():

    # gLite
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
        os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
    glib.export('GLITE_LOCATION_LOG');
    if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
        os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
    glib.export('GLITE_LOCATION_TMP');
    if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
        os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)

    glib.export('JAVA_HOME')

    # bin and lib paths
    glib.addEnvPath("PATH","%s/externals/bin:%s/bin:%s/bin" % (os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION'],os.environ['JAVA_HOME']))

    # Set environment
    glib.setUserEnv()
    

#################################################################
# main
#
# Main function
#################################################################
if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid()==0:
        glib.printErrorMessage("\nThis script must be run as root.\n")
        sys.exit(1)

    # Get an instance of the library class
    glib = gLib()
    glib.verbose = 0
    verboseCLI = 0
    
    #-------------------------------------------------------------------------------    
    # Load parameters
    #-------------------------------------------------------------------------------    

    # Get an instance of the ConfigParams class
    params = ConfigParams()

    # set default parameters
    loadDefaultParameters(params)    
        
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', 'configureExternalServices', 'start', 'stop', 'status', 'siteconfig=', 'subservice', 'configure', 'verbose', 'removeService='])
        for o, a in opts:
            
            if o == "--verbose":
                verboseCLI = 1
                glib.verbose = 1
                
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    
    except getopt.GetoptError:
        pass
    
    # load rgma common and rgma gin configuration parameters
    configFiles = ["%s/../glite-rgma-common.cfg.xml" % glib.getScriptPath(), 
                   "%s/../glite-rgma-servicetool.cfg.xml" % glib.getScriptPath()]
        
    if glib.loadConfiguration(configFiles,params):
        glib.printErrorMessage("Problems while loading configuration file.")
        sys.exit(1)
                        
    #-------------------------------------------------------------------------------    
    # set verbosity
    #-------------------------------------------------------------------------------    
    verbose = 0
    if verboseCLI:
        verbose = 1
    else:
        if params.has_key('glite.installer.verbose'):
            if params['glite.installer.verbose'] == "true":
                verbose = 1
    
    glib.verbose = verbose
        
    #-------------------------------------------------------------------------------    
    # Set up the environment
    #-------------------------------------------------------------------------------    
    set_env()
          
    #-------------------------------------------------------------------------------    
    # Instantiate the service classes
    #-------------------------------------------------------------------------------    
    service = glite_rgma_servicetool()
    service.verbose = verbose
        
    #-------------------------------------------------------------------------------    
    # Command line opts if any
    #-------------------------------------------------------------------------------    
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', 'configureExternalServices', 'start', 'stop', 'status', 'siteconfig=', 'subservice', 'configure', 'verbose', 'removeService='])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    #-------------------------------------------------------------------------------    
    # Check cli options
    #-------------------------------------------------------------------------------    
    subservice     = 0
    configure      = 0
    start          = 0
    
    for o, a in opts:
    
        # show help
        if o in ("-h", "--help"):        
            service.usage()
            sys.exit(0)
    
        # show version
        if o in ("-v", "--version"):        
            service.showVersion()
            sys.exit(0)
    
        # check configuration
        if o in ("-c", "--checkconf"):
            if service.showConfiguration():
                sys.exit(1)
                
            sys.exit(0)
    
        # configure a non glite service for servicetool
        if o == "--configureExternalServices":
            
            # load rgma common and rgma gin configuration parameters
            configFiles = ["%s/../glite-rgma-common.cfg.xml" % glib.getScriptPath(), 
                           "%s/../glite-rgma-servicetool.cfg.xml" % glib.getScriptPath(),
                           "%s/../glite-rgma-servicetool-externalServices.cfg.xml" % glib.getScriptPath()]
        
            if glib.loadConfiguration(configFiles,params):
                glib.printErrorMessage("Problems while loading configuration file.")
                sys.exit(1)
                
            if service.addExternalServices():
                print 
                glib.printErrorMessage("An unrecoverable error occurred while adding the configuration of external services to %s" % service.friendly_name)
                sys.exit(1)
                
            print "\nConfiguration of external services successfully completed\n"
            sys.exit(0)
        
        # start servicetool        
        if o == "--start":            
            start = 1
    
        # stop servicetool
        if o == "--stop":            
            errorcode = service.stop()
            if errorcode == 0:
                sys.exit(0)
            else:
                if errorcode == -1:
                    # service was not running
                    sys.exit(0)
                else:
                    glib.printErrorMessage("An error occurred while stopping the %s service" % service.friendly_name)
                    sys.exit(1)
    
        # show status of servicetool
        if o == "--status":            
            returncode = service.status()
            if returncode == 0:
                sys.exit(0)
            else:
                sys.exit(1)
            
        # use script as subservice of other service
        if o == "--subservice":            
            subservice = 1
            params['rgma.servicetool.run_as_subservice'] = "true"
    
        # configure
        if o == "--configure":
            configure = 1
            
        # remove service
        if o == "--removeService":
            serviceName = a
            if service.removeService(serviceName):
                sys.exit(1)
            sys.exit(0)

    #-------------------------------------------------------------------------------    
    # Configure
    #-------------------------------------------------------------------------------    
    if configure==1:
        
        #-------------------------------------------------------------------------------    
        # Print configuration parameters
        #-------------------------------------------------------------------------------    
        if verbose:
            glib.print_params(params)

        #-------------------------------------------------------------------------------
        # print default statements
        #-------------------------------------------------------------------------------    
        if subservice==0:
            service.copyright()
            service.showVersion()
       
        #-------------------------------------------------------------------------------
        # stop servicetool first in case it is running and 
        # if script is not called as subservice
        #-------------------------------------------------------------------------------    
        if subservice==0:
            service.stop()

        # print banner
        service.banner()
         
        #-------------------------------------------------------------------------------
        # Configure the Service Publisher
        #-------------------------------------------------------------------------------    
        if service.configure() != 0:
            glib.printErrorMessage("\nAn unrecoverable error occurred while configuring the %s service" % service.friendly_name)
            sys.exit(1)
        else:
            print "\n%s configuration successfully completed\n" % service.friendly_name            
            if subservice==0:
                print "To start the %s please run" % service.friendly_name
                print "    %s/etc/config/scripts/glite-rgma-servicetool-config.py --start" % os.environ['GLITE_LOCATION']
            sys.exit(0)                            
    
    else:

        #-------------------------------------------------------------------------------    
        # Start
        #-------------------------------------------------------------------------------    
        if start==1:

            returncode = service.start()
            if  returncode == 0:
                print "\n%s successfully started\n" % service.friendly_name

                if subservice==0:
                    print "To check if the %s is running correctly you can run " % service.friendly_name
                    print "    %s/etc/config/scripts/glite-rgma-servicetool-config.py --status\n" % os.environ['GLITE_LOCATION']
                
                sys.exit(0)
            else:
                if returncode == -1:
                    # nothing to be done (not activated)
                    sys.exit(0)
                else:
                    glib.printErrorMessage("An error occurred while starting the %s service" % service.friendly_name)
                    sys.exit(1)

        else:
            # ------------------------------------------------------------
            # No option given
            # ------------------------------------------------------------
            service.usage()
            sys.exit(0)        
        
